import time
import subprocess
import yaml

from charmhelpers.core import host, hookenv, unitdata
from charms.reactive import (
    when, when_not, when_any, set_state, remove_state
)
from charms.layer import snap
import charms.promreg


charmconfig = hookenv.config()
if charmconfig.get('use_mlog', False):
    SNAP = 'grok-exporter-mlog'
else:
    SNAP = 'grok-exporter'
SVC_NAME = 'snap.{}.grok-exporter'.format(SNAP)
PORT_DEF = 9144
CONFIG_PATH = '/var/snap/{}/current/etc/config.yml'.format(SNAP)


def grok_exporter_port():
    config = charmconfig.get('config', '')
    if not config:
        return PORT_DEF

    try:
        config_yaml = yaml.safe_load(config)
        return config_yaml['server']['port']
    # if the config isn't proper YAML, or if server.port isn't defined, just
    # return the default port
    except: # noqa E722
        return PORT_DEF


def check_port(new_port):
    unitdata_key = 'grok_exporter.port'
    kv = unitdata.kv()
    existing_port = kv.get(unitdata_key)
    if existing_port != new_port:
        hookenv.open_port(new_port)
        if existing_port:  # Dont try to close non existing ports
            hookenv.close_port(existing_port)
            charms.promreg.deregister(None, existing_port)
        kv.set(unitdata_key, new_port)

    # Register the port, even if this is a re-registration
    charms.promreg.register(None, new_port)


@when_not('grok-exporter.installed')
def install_packages():
    hookenv.status_set('maintenance', 'Installing software')
    channel = charmconfig.get('snap_channel', 'stable')
    snap.install(SNAP, channel=channel, force_dangerous=False)

    # TODO: remove 3 calls below when we get autoconnect assertion
    subprocess.check_call(['snap', 'connect', '{}:log-observe'.format(SNAP)])
    subprocess.check_call(['snap', 'disable', SNAP])
    subprocess.check_call(['snap', 'enable', SNAP])

    set_state('grok-exporter.installed')
    set_state('grok-exporter.do-reconfig')


@when('grok-exporter.installed')
@when_any('grok-exporter.do-reconfig', 'config.changed.config',
          'config.changed.promreg_authtoken', 'config.changed.promreg_url')
def write_config():
    config = charmconfig.get('config', '')
    if config:
        with open(CONFIG_PATH, 'w') as f:
            f.write(config)
    check_port(grok_exporter_port())
    remove_state('grok-exporter.do-reconfig')
    set_state('grok-exporter.do-restart')


@when('config.changed.snap_channel')
def snap_channel_changed():
    remove_state('grok-exporter.installed')


@when('grok-exporter.do-restart')
def restart_grok_exporter():
    remove_state('grok-exporter.do-restart')
    if not host.service_running(SVC_NAME):
        hookenv.log('Starting {}...'.format(SVC_NAME))
        host.service_start(SVC_NAME)
    else:
        hookenv.log('Restarting {}, config file changed...'.format(SVC_NAME))
        host.service_restart(SVC_NAME)
    set_state('grok-exporter.started')

    time.sleep(2)  # Wait for restart to happen

    if not host.service_running(SVC_NAME):
        msg = 'Service down, manual intervention needed'
        hookenv.status_set('blocked', msg)
        hookenv.log(msg)
    else:
        hookenv.log('Service started fine')
        hookenv.status_set('active', 'Service up')


# Relations
@when('grok-exporter.started')
@when('prometheus_client.available')
@when_not('grok-exporter.prometheus_relation_configured')
def configure_prometheus_relation(target):
    target.configure(grok_exporter_port())
    set_state('grok-exporter.prometheus_relation_configured')
